"""
This module decorates the django templatize function to parse haml templates
before the translation utility extracts tags from it.
"""

from django.utils.translation import trans_real
import hamlpy


def decorate_templatize(func):
	def templatize(src, origin=None):
		hamlParser = hamlpy.Compiler()
		html = hamlParser.process(src)
		return func(html, origin)
	
	return templatize

trans_real.templatize = decorate_templatize(trans_real.templatize)




#!/usr/bin/env python
from nodes import RootNode, FilterNode, HamlNode, create_node

class Compiler:
    def process(self, raw_text):
        split_text = raw_text.strip().split('\n')
        return self.process_lines(split_text)

    def process_lines(self, haml_lines):
        root = RootNode()
        line_iter = iter(haml_lines)

        for line_number, line in enumerate(line_iter):
            node_lines = line

            # Check for multi-line string when parent is not a FilterNode
            if not isinstance(root.parent( HamlNode(line) ), FilterNode):
                if line.count('{') - line.count('}') == 1:
                    start_multiline=line_number # For exception handling

                    while line.count('{') - line.count('}') != -1:
                        try:
                            line = line_iter.next()
                        except StopIteration:
                            raise Exception('No closing brace found for multi-line HAML beginning at line %s' % (start_multiline+1))
                        node_lines += line
            haml_node = create_node(node_lines)
            root.add_node(haml_node)
        return root.render()

def convert_files():
    import sys
    import codecs
    
    if len(sys.argv) < 2:
        print "Specify the input file as the first argument."
    else: 
        infile = sys.argv[1]
        haml_lines = codecs.open(infile, 'r', encoding='utf-8').read().splitlines()

        compiler = Compiler()
        output = compiler.process_lines(haml_lines)
        
        if len(sys.argv) == 3:
            outfile = codecs.open(sys.argv[2], 'w', encoding='utf-8')
            outfile.write(output)
        else:
            print output
    
    
if __name__ == '__main__':
    convert_files()




# haml-watcher.py
# Author: Christian Stefanescu (st.chris@gmail.com)
#
# Watch a folder for files with the given extensions and call the HamlPy
# compiler if the modified time has changed since the last check.
from time import gmtime, strftime
import sys
import codecs
import os
import os.path
import time
from hamlpy import Compiler

EXTENSIONS = ['.haml', '.hamlpy']    # watched extensions
CHECK_INTERVAL = 3          # in seconds
DEBUG = False               # print file paths when a file is compiled

# dict of compiled files [fullpath : timestamp]
compiled = dict()

def watched_extension(extension):
    """Return True if the given extension is one of the watched extensions"""
    for ext in EXTENSIONS:
        if extension.endswith(ext):
            return True
    return False

def watch_folder():
    """Main entry point. Expects one or two arguments (the watch folder + optional destination folder)."""
    argv_len = len(sys.argv)
    if argv_len in (2, 3):
        folder = os.path.realpath(sys.argv[1])
        destination = os.path.realpath(argv_len == 3 and os.path.realpath(sys.argv[2]) or folder)
        
        print "Watching %s at refresh interval %s seconds" % (folder,CHECK_INTERVAL)
        while True:
            try:
                _watch_folder(folder, destination)
                time.sleep(CHECK_INTERVAL)
            except KeyboardInterrupt:
                # allow graceful exit (no stacktrace output)
                sys.exit(0)
                pass
    else:
        print "Usage: hamlpy-watcher.py <watch_folder> [destination_folder]"

def _watch_folder(folder, destination):
    """Compares "modified" timestamps against the "compiled" dict, calls compiler
    if necessary."""
    for dirpath, dirnames, filenames in os.walk(folder):
        for filename in filenames:
            # Ignore filenames starting with ".#" for Emacs compatibility
            if watched_extension(filename) and not filename.startswith('.#'):
                fullpath = os.path.join(dirpath, filename)
                subfolder = os.path.relpath(dirpath, folder)
                mtime = os.stat(fullpath).st_mtime
				
                # Create subfolders in target directory if they don't exist
                compiled_folder = os.path.join(destination, subfolder)
                if not os.path.exists(compiled_folder):
                    os.makedirs(compiled_folder)
				
                compiled_path = _compiled_path(compiled_folder, filename)
                if (not fullpath in compiled or
                    compiled[fullpath] < mtime or
                    not os.path.isfile(compiled_path)):
                    compile_file(fullpath, compiled_path)
                    compiled[fullpath] = mtime

def _compiled_path(destination, filename):
    return os.path.join(destination, filename[:filename.rfind('.')] + '.html')

def compile_file(fullpath, outfile_name):
    """Calls HamlPy compiler."""
    print '%s %s -> %s' % ( strftime("%H:%M:%S", gmtime()), fullpath, outfile_name )
    try:
        if DEBUG:
            print "Compiling %s -> %s" % (fullpath, outfile_name)
        haml_lines = codecs.open(fullpath, 'r', encoding='utf-8').read().splitlines()
        compiler = Compiler()
        output = compiler.process_lines(haml_lines)
        outfile = codecs.open(outfile_name, 'w', encoding='utf-8')
        outfile.write(output)
    except Exception, e:
        # import traceback
        print "Failed to compile %s -> %s\nReason:\n%s" % (fullpath, outfile_name, e)
        # print traceback.print_exc()

if __name__ == '__main__':
    watch_folder()



import re
import sys
from StringIO import StringIO

from elements import Element

from pygments import highlight
from pygments.formatters import HtmlFormatter
from pygments.lexers import guess_lexer, guess_lexer_for_filename

ELEMENT = '%'
ID = '#'
CLASS = '.'
DOCTYPE = '!!!'

HTML_COMMENT = '/'
CONDITIONAL_COMMENT = '/['
HAML_COMMENTS = ['-#', '=#']

VARIABLE = '='
TAG = '-'

COFFEESCRIPT_FILTERS = [':coffeescript', ':coffee']
JAVASCRIPT_FILTER = ':javascript'
CSS_FILTER = ':css'
STYLUS_FILTER = ':stylus'
PLAIN_FILTER = ':plain'
PYTHON_FILTER = ':python'
CDATA_FILTER = ':cdata'
PYGMENTS_FILTER = ':highlight'

ELEMENT_CHARACTERS = (ELEMENT, ID, CLASS)

HAML_ESCAPE = '\\'

def create_node(haml_line):
    stripped_line = haml_line.strip()
    
    if not stripped_line:
        return None
        
    if stripped_line[0] == HAML_ESCAPE:
        return HamlNode(haml_line.replace(HAML_ESCAPE, '', 1))
        
    if stripped_line.startswith(DOCTYPE):
        return DoctypeNode(haml_line)
        
    if stripped_line[0] in ELEMENT_CHARACTERS:
        return ElementNode(haml_line)
    
    if stripped_line[0:len(CONDITIONAL_COMMENT)] == CONDITIONAL_COMMENT:
        return ConditionalCommentNode(haml_line)
        
    if stripped_line[0] == HTML_COMMENT:
        return CommentNode(haml_line)
    
    for comment_prefix in HAML_COMMENTS:
        if stripped_line.startswith(comment_prefix):
            return HamlCommentNode(haml_line)
    
    if stripped_line[0] == VARIABLE:
        return VariableNode(haml_line)

    if stripped_line[0] == TAG:
        return TagNode(haml_line)
    
    if stripped_line == JAVASCRIPT_FILTER:
        return JavascriptFilterNode(haml_line)
    
    if stripped_line in COFFEESCRIPT_FILTERS:
        return CoffeeScriptFilterNode(haml_line)
        
    if stripped_line == CSS_FILTER:
        return CssFilterNode(haml_line)
    
    if stripped_line == STYLUS_FILTER:
        return StylusFilterNode(haml_line)

    if stripped_line == PLAIN_FILTER:
        return PlainFilterNode(haml_line)
        
    if stripped_line == PYTHON_FILTER:
        return PythonFilterNode(haml_line)
    
    if stripped_line == CDATA_FILTER:
        return CDataFilterNode(haml_line)
		
	if stripped_line == PYGMENTS_FILTER:
		return PygmentsFilterNode(haml_line)
    
    return HamlNode(haml_line)

class RootNode:
    
    def __init__(self):
        self.indentation = -1
        self.internal_nodes = []
    
    def parent(self,node):
        if (node == None):
            return None

        if (self._should_go_inside_last_node(node)):
            ret = self.internal_nodes[-1].parent(node)
            return ret
        else:
            return self

    def add_node(self, node):
        if (node == None):
            return
        
        if (self._should_go_inside_last_node(node)):
            self.internal_nodes[-1].add_node(node)
        else:
            self.internal_nodes.append(node)
    
    def _should_go_inside_last_node(self, node):
        return len(self.internal_nodes)>0 and (node.indentation > self.internal_nodes[-1].indentation
            or (node.indentation == self.internal_nodes[-1].indentation and self.internal_nodes[-1].should_contain(node)))
    
    def render(self):
        return self.render_internal_nodes()
    
    def render_internal_nodes(self):
        return ''.join([node.render() for node in self.internal_nodes])
    
    def has_internal_nodes(self):
        return len(self.internal_nodes) > 0
    
    def should_contain(self, node):
        return False
      
        
class HamlNode(RootNode):
    
    def __init__(self, haml):
        RootNode.__init__(self)
        self.haml = haml.strip()
        self.raw_haml = haml
        self.indentation = (len(haml) - len(haml.lstrip()))
        self.spaces = ''.join(haml[0] for i in range(self.indentation))
    
    def render(self):
        return ''.join([self.spaces, self.haml, '\n', self.render_internal_nodes()])

    def __repr__(self):
        return '(%s) %s' % (self.__class__, self.haml)


class ElementNode(HamlNode):
    def __init__(self, haml):
        HamlNode.__init__(self, haml)
        self.django_variable = False
    
    def render(self):
        return self._render_tag()
    
    def _render_tag(self):
        element = Element(self.haml)
        self.django_variable = element.django_variable
        return self._generate_html(element)
        
    def _generate_html(self, element):        
        if self.indentation > 0:
            result = "%s<%s" % (self.spaces, element.tag) 
        else:
            result = "<%s" % element.tag 

        if element.id:
            result += " id='%s'" % element.id 
        if element.classes:
            result += " class='%s'" % element.classes 
        if element.attributes:
            result += ' ' + element.attributes            
            
        content = self._render_tag_content(element.inline_content)
        
        if element.self_close and not content:
            result += " />\n"
        else:
            result += ">%s</%s>\n" % (content, element.tag)
        
        return result
    
    def _render_tag_content(self, current_tag_content):
        if self.has_internal_nodes():
            current_tag_content = '\n' + self.render_internal_nodes() + self.spaces
        if current_tag_content == None:
            current_tag_content = ''
        if self.django_variable:
            current_tag_content = "{{ " + current_tag_content.strip() + " }}"
        current_tag_content = re.sub(r'#\{([a-zA-Z0-9\.\_]+)\}', r'{{ \1 }}', current_tag_content)
        return current_tag_content


class CommentNode(HamlNode):
    
    def render(self):
        content = ''
        if self.has_internal_nodes():
            content = '\n' + self.render_internal_nodes()
        else:
            content = self.haml.lstrip(HTML_COMMENT).strip() + ' '
        
        return "<!-- %s-->\n" % content

class ConditionalCommentNode(HamlNode):
    
    def render(self):
        conditional = self.haml[1: self.haml.index(']')+1 ]
        content = ''
        content = content + self.haml[self.haml.index(']')+1:]
        if self.has_internal_nodes():
            content = '\n' + self.render_internal_nodes()
        return "<!--%s>%s<![endif]-->" % (conditional, content)
        

class DoctypeNode(HamlNode):
    
    def render(self):
        doctype = self.haml.lstrip(DOCTYPE).strip()
        
        if doctype == "":
            content = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">'
        if doctype == "Strict":
            content = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">'
        if doctype == "Frameset":
            content = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Frameset//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-frameset.dtd">'
        if doctype == "5":
            content = '<!DOCTYPE html>'
        if doctype == "1.1":
            content = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">'
        
        return "%s\n" % content

class HamlCommentNode(HamlNode):
    
    def render(self):
        return ''


class VariableNode(ElementNode):
    def __init__(self, haml):
        ElementNode.__init__(self, haml)
        self.django_variable = True
    
    def render(self):
        tag_content = self.haml.lstrip(VARIABLE)
        return "%s%s\n" % (self.spaces, self._render_tag_content(tag_content))


class TagNode(HamlNode):
    self_closing = {'for':'endfor',
                    'if':'endif',
                    'ifchanged':'endifchanged',
                    'ifequal':'endifequal',
                    'ifnotequal':'endifnotequal',
                    'block':'endblock',
                    'filter':'endfilter',
                    'autoescape':'endautoescape',
                    'with':'endwith',
                    'blocktrans': 'endblocktrans',
                    'spaceless': 'endspaceless',
                    'comment': 'endcomment',
                    'cache': 'endcache',
                    'localize': 'endlocalize',
                    'compress': 'endcompress'}
    may_contain = {'if':['else', 'elif'], 
                   'ifchanged':'else',
                   'ifequal':'else',
                   'ifnotequal':'else',
                   'for':'empty', 
                   'with':'with'}
    
    def __init__(self, haml):
        HamlNode.__init__(self, haml)
        self.tag_statement = self.haml.lstrip(TAG).strip()
        self.tag_name = self.tag_statement.split(' ')[0]
        
        if (self.tag_name in self.self_closing.values()):
            raise TypeError("Do not close your Django tags manually.  It will be done for you.")
    
    def render(self):
        internal = self.render_internal_nodes()
        output = "%s{%% %s %%}\n%s" % (self.spaces, self.tag_statement, internal)
        if (self.tag_name in self.self_closing.keys()):
            output += '%s{%% %s %%}\n' % (self.spaces, self.self_closing[self.tag_name])
        return output
    
    def should_contain(self, node):
        return isinstance(node,TagNode) and node.tag_name in self.may_contain.get(self.tag_name,'')


class FilterNode(HamlNode):
  def add_node(self, node):
      if (node == None):
          return
      else:
          self.internal_nodes.append(node)


class PlainFilterNode(FilterNode):
    def render(self):
        if self.internal_nodes:
            first_indentation = self.internal_nodes[0].indentation
        return "".join([node.raw_haml[first_indentation:] + '\n' for node in self.internal_nodes])


class PythonFilterNode(FilterNode):
    def render(self):
        code = compile("".join([node.raw_haml.strip() + '\n' for node in self.internal_nodes]), "", "exec")
        
        buffer = StringIO()
        sys.stdout = buffer
        exec code
        # restore the original stdout
        sys.stdout = sys.__stdout__
        return buffer.getvalue()


class JavascriptFilterNode(FilterNode):
    def render(self):
        output = '<script type=\'text/javascript\'>\n// <![CDATA[\n'
        output += "".join((''.join((node.spaces, node.haml,'\n')) for node in self.internal_nodes))
        output += '// ]]>\n</script>\n'
        return output
        
        
class CoffeeScriptFilterNode(FilterNode):
    def render(self):
        output = '<script type=\'text/coffeescript\'>#<![CDATA[\n'
        output += ''.join([node.raw_haml for node in self.internal_nodes])
        output += '\n#]]></script>'
        return output


class CssFilterNode(FilterNode):
    def render(self):
        output = '<style type=\'text/css\'>\n/*<![CDATA[*/\n'
        output += "".join((''.join((node.spaces, node.haml,'\n')) for node in self.internal_nodes))
        output += '/*]]>*/\n</style>\n'
        return output


class StylusFilterNode(FilterNode):
    def render(self):
        output = '<style type=\'text/stylus\'>\n/*<![CDATA[*/\n'
        first_indentation = self.internal_nodes[0].indentation
        output += '\n'.join([node.raw_haml[first_indentation:] for node in self.internal_nodes])
        output += '\n/*]]>*/\n</style>\n'
        return output


class CDataFilterNode(FilterNode):
    def render(self):
        output = self.spaces + '<![CDATA[\n'
        output += "".join((''.join((node.spaces, node.haml,'\n')) for node in self.internal_nodes))
        output += self.spaces + ']]>\n'
        return output
		
class PygmentsFilterNode(FilterNode):
    def render(self):
        output = self.spaces
        output += highlighter(self.haml, guess_lexer(self.haml), HtmlFormatter())	
        return output
